@@ -8,13 +8,13 @@
 #include "kweathercore_p.h"
 #include "pendingweatherforecast_p.h"
 #include <QJsonArray>
+#include <QJsonObject>
+
 namespace KWeatherCore
 {
-class DailyWeatherForecast::DailyWeatherForecastPrivate
+class DailyWeatherForecastPrivate : public QSharedData
 {
 public:
-    bool isValid = true;
-
     double maxTemp = std::numeric_limits<double>::lowest();
     double minTemp = std::numeric_limits<double>::max();
     double precipitation = 0; // mm
@@ -28,25 +28,21 @@ public:
     std::vector<HourlyWeatherForecast> hourlyWeatherForecast;
 };
 DailyWeatherForecast::~DailyWeatherForecast() = default;
-DailyWeatherForecast::DailyWeatherForecast(DailyWeatherForecast &&other) = default;
-DailyWeatherForecast &DailyWeatherForecast::operator=(DailyWeatherForecast &&other) = default;
+DailyWeatherForecast::DailyWeatherForecast(DailyWeatherForecast &&other) noexcept = default;
+DailyWeatherForecast &DailyWeatherForecast::operator=(const DailyWeatherForecast &other) = default;
+DailyWeatherForecast &DailyWeatherForecast::operator=(DailyWeatherForecast &&other) noexcept = default;
 DailyWeatherForecast::DailyWeatherForecast()
-    : d(std::make_unique<DailyWeatherForecastPrivate>())
+    : d(new DailyWeatherForecastPrivate)
 {
 }
 DailyWeatherForecast::DailyWeatherForecast(const QDate &date)
-    : d(std::make_unique<DailyWeatherForecastPrivate>())
+    : d(new DailyWeatherForecastPrivate)
 {
     d->date = date;
-    d->isValid = false;
-}
-DailyWeatherForecast::DailyWeatherForecast(const DailyWeatherForecast &other)
-    : d(std::make_unique<DailyWeatherForecastPrivate>())
-{
-    *d = *other.d;
 }
+DailyWeatherForecast::DailyWeatherForecast(const DailyWeatherForecast &other) = default;
 
-QJsonObject DailyWeatherForecast::toJson()
+QJsonObject DailyWeatherForecast::toJson() const
 {
     QJsonObject obj;
     QJsonArray hourlyArray;
@@ -81,13 +77,9 @@ DailyWeatherForecast DailyWeatherForecast::fromJson(const QJsonObject &obj)
     for (int i = 0; i < array.size(); i++) {
         hourlyVec.push_back(HourlyWeatherForecast::fromJson(array.at(i).toObject()));
     }
-    ret.setHourlyWeatherForecast(hourlyVec);
+    ret.setHourlyWeatherForecast(std::move(hourlyVec));
     return ret;
 }
-bool DailyWeatherForecast::isValid() const
-{
-    return d->isValid;
-}
 void DailyWeatherForecast::setMaxTemp(double maxTemp)
 {
     d->maxTemp = maxTemp;
@@ -172,22 +164,17 @@ const std::vector<HourlyWeatherForecast> &DailyWeatherForecast::hourlyWeatherFor
 {
     return d->hourlyWeatherForecast;
 }
-void DailyWeatherForecast::setHourlyWeatherForecast(const std::vector<HourlyWeatherForecast> &forecast)
-{
-    d->hourlyWeatherForecast = forecast;
-}
 void DailyWeatherForecast::setHourlyWeatherForecast(std::vector<HourlyWeatherForecast> &&forecast)
 {
     d->hourlyWeatherForecast = std::move(forecast);
 }
 
-DailyWeatherForecast &DailyWeatherForecast::operator+=(const HourlyWeatherForecast &forecast)
+DailyWeatherForecast &DailyWeatherForecast::operator+=(HourlyWeatherForecast &&forecast)
 {
-    if (isValid()) {
+    if (!d->date.isValid()) {
         setDate(forecast.date().date());
         setWeatherDescription(forecast.weatherDescription());
         setWeatherIcon(forecast.weatherIcon());
-        d->isValid = false;
     }
 
     if (date().daysTo(forecast.date().date()) == 0) {
@@ -204,7 +191,7 @@ DailyWeatherForecast &DailyWeatherForecast::operator+=(const HourlyWeatherForeca
         setMinTemp(std::min(minTemp(), forecast.temperature()));
     }
 
-    d->hourlyWeatherForecast.push_back(forecast);
+    d->hourlyWeatherForecast.push_back(std::move(forecast));
     return *this;
 }
 
@@ -217,11 +204,6 @@ bool DailyWeatherForecast::operator<(const DailyWeatherForecast &forecast) const
 {
     return date() < forecast.date();
 }
-DailyWeatherForecast &DailyWeatherForecast::operator=(const DailyWeatherForecast &other)
-{
-    *d = *other.d;
-    return *this;
-}
 }
 
 #include "moc_dailyweatherforecast.cpp"
